from __future__ import absolute_import
from http.client import HTTPSConnection, HTTPResponse
from gzip import decompress
import json, time

from celery import shared_task

from .urlgen import *
from .models import *

# TODO: put in settings
conn_info = {
	'key_file':  '/home/robbmi/myprograms/sscert/sscert.key',
	'cert_file': '/home/robbmi/myprograms/sscert/sscert.crt',
	'host': 'api.stackexchange.com',
	'port': 443,
}

def stack_conn(conn_info):
	""" returns an HTTPSConnection Objest to api.stackexchange.com, the key and cert files have been loaded """
	return HTTPSConnection(conn_info['host'], conn_info['port'], conn_info['key_file'], conn_info['cert_file'])

def get_response(url):
	""" resturns the raw response body """
	print(url)
	conn = stack_conn(conn_info)
	conn.request('GET', url)
	response = conn.getresponse()
	ret = None
	if response.status == 200:
		ret = response.read()
	conn.close()
	return ret
	
def response_parser(raw_response):
	""" unzip it, bytesObj to string, decode JSON """
	data = json.loads( (decompress( raw_response )).decode("utf-8") )
	return data['items']

@shared_task
def user_info_task(user_id):
#	return __r
	user = StackExchangeUser.objects.get(pk=user_id)
	token = AccessToken(user)
	url = URL(StackOverflow(), AppKey())
	response = get_response(url(Me(), token))
	if response:
		data = response_parser(response)
		user.so_reputation = data[0]['reputation']
		user.save()
		return data[0]['reputation']
	return None


@shared_task
def new_questions_task(user_id):
	#return json.loads(__q)
	user = StackExchangeUser.objects.get(pk=user_id)

	if not hasattr(user, 'newquestions'):
		questions = NewQuestions(for_user=user)
		last_checked = int(time.time())
	else:
		questions = user.newquestions
		# convert to unix timestamp
		last_checked = int(questions.last_checked.timestamp())

	token = AccessToken(user)
	url = URL(StackOverflow(), AppKey())
	response = get_response(url(MyNewQuestions(str(last_checked)), token))
	if response:
		data = response_parser(response)
		if len(data):
			questions.newest_id = data[0]['question_id']
			questions.save()
		return data
	return None



__q = '[{"tags":["java","json","spring","spring-boot","thymeleaf"],"owner":{"reputation":228,"user_id":1251350,"user_type":"registered","profile_image":"http://i.stack.imgur.com/oFjTi.jpg?s=128&g=1","display_name":"Faraj Farook","link":"http://stackoverflow.com/users/1251350/faraj-farook"},"is_answered":false,"view_count":2,"answer_count":0,"score":0,"last_activity_date":1427827281,"creation_date":1427827281,"question_id":29376071,"link":"http://stackoverflow.com/questions/29376071/dandelionexception-the-file-resources-lib-icheck-icheck-jquery-json-is-wrongl","title":"DandelionException: The file &#39;resources/lib/icheck/icheck.jquery.json&#39; is wrongly formatted. Please correct it before continuing"},{"tags":["php","arrays","multidimensional-array"],"owner":{"reputation":11,"user_id":4726935,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/9469453e50cd05d0c46e55bad96dc2d2?s=128&d=identicon&r=PG&f=1","display_name":"YouNoobsDie","link":"http://stackoverflow.com/users/4726935/younoobsdie"},"is_answered":false,"view_count":5,"answer_count":0,"score":0,"last_activity_date":1427827264,"creation_date":1427827264,"question_id":29376059,"link":"http://stackoverflow.com/questions/29376059/how-can-i-convert-1d-array-from-numbers-to-2d-array","title":"How can I convert 1D array from numbers to 2D array?"},{"tags":["javascript","jquery","html","css","animation"],"owner":{"reputation":135,"user_id":2313464,"user_type":"registered","accept_rate":83,"profile_image":"http://i.stack.imgur.com/KZzwa.jpg?s=128&g=1","display_name":"Dennington-bear","link":"http://stackoverflow.com/users/2313464/dennington-bear"},"is_answered":false,"view_count":4,"answer_count":0,"score":0,"last_activity_date":1427827251,"creation_date":1427827251,"question_id":29376053,"link":"http://stackoverflow.com/questions/29376053/changing-css-animation-in-javascript","title":"Changing css animation in javascript"},{"tags":["php","mysql","database","select","pdo"],"owner":{"reputation":157,"user_id":2584871,"user_type":"registered","accept_rate":75,"profile_image":"https://www.gravatar.com/avatar/014c8c15f318a2d7d5e59a22273d2af5?s=128&d=identicon&r=PG","display_name":"jdc987","link":"http://stackoverflow.com/users/2584871/jdc987"},"is_answered":false,"view_count":2,"answer_count":0,"score":0,"last_activity_date":1427827234,"creation_date":1427827234,"question_id":29376044,"link":"http://stackoverflow.com/questions/29376044/php-mysql-code-working-on-local-host-with-mysql-version-5-5-34-but-not-working","title":"PHP/MySQL code working on local host with MySQL version 5.5.34, but not working on server with MySQL version 5.5.40-36. What is wrong with the code?"},{"tags":["c"],"owner":{"reputation":24,"user_id":1886669,"user_type":"registered","accept_rate":43,"profile_image":"https://www.gravatar.com/avatar/ba43c38cd84ee1bacc21e312b29ea102?s=128&d=identicon&r=PG","display_name":"Tyler","link":"http://stackoverflow.com/users/1886669/tyler"},"is_answered":false,"view_count":6,"answer_count":1,"score":0,"last_activity_date":1427827285,"creation_date":1427827229,"question_id":29376042,"link":"http://stackoverflow.com/questions/29376042/declaring-int-variables-causes-segmentation-fault","title":"Declaring Int variables causes segmentation fault?"},{"tags":["java","jsf","primefaces","jsf-2.2"],"owner":{"reputation":64,"user_id":276149,"user_type":"registered","accept_rate":44,"profile_image":"https://www.gravatar.com/avatar/2ca5f3327a7f319e9106631ac91c3fd3?s=128&d=identicon&r=PG","display_name":"karki","link":"http://stackoverflow.com/users/276149/karki"},"is_answered":false,"view_count":3,"answer_count":0,"score":0,"last_activity_date":1427827217,"creation_date":1427827217,"question_id":29376038,"link":"http://stackoverflow.com/questions/29376038/setpropertyactionlistener-not-fired-the-setter-method-of-target","title":"setPropertyActionListener not fired the setter method of target"},{"tags":["javascript","jquery","jstree"],"owner":{"reputation":1,"user_id":1639687,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/c8d4d31b1a678c60455c351ced6c3c07?s=128&d=identicon&r=PG","display_name":"eng.r","link":"http://stackoverflow.com/users/1639687/eng-r"},"is_answered":false,"view_count":2,"answer_count":0,"score":0,"last_activity_date":1427827205,"creation_date":1427827205,"question_id":29376033,"link":"http://stackoverflow.com/questions/29376033/jstree-create-and-rename-new-node","title":"jstree - create and rename new node"},{"tags":["java","json","arraylist"],"owner":{"reputation":52,"user_id":1367670,"user_type":"registered","accept_rate":81,"profile_image":"https://www.gravatar.com/avatar/34e4d54484c4c5c624a6fc6e5b1e7ca6?s=128&d=identicon&r=PG","display_name":"Sandy","link":"http://stackoverflow.com/users/1367670/sandy"},"is_answered":false,"view_count":4,"answer_count":0,"score":0,"last_activity_date":1427827199,"creation_date":1427827199,"question_id":29376032,"link":"http://stackoverflow.com/questions/29376032/convert-json-to-listliststring-in-java","title":"Convert JSON to List&lt;List&lt;String&gt;&gt; in JAVA"},{"tags":["java","parsing","line","reader"],"owner":{"reputation":18,"user_id":3938589,"user_type":"registered","accept_rate":20,"profile_image":"https://www.gravatar.com/avatar/138eddae905a73bc28820e245dcc22d8?s=128&d=identicon&r=PG&f=1","display_name":"Rez88","link":"http://stackoverflow.com/users/3938589/rez88"},"is_answered":false,"view_count":8,"answer_count":0,"score":0,"last_activity_date":1427827166,"creation_date":1427827166,"question_id":29376024,"link":"http://stackoverflow.com/questions/29376024/finding-string-in-a-list-of-text-files-in-a-directory-java-want-to-locate-line","title":"Finding String in a list of text files in a directory Java ..want to locate line number."},{"tags":["java","jvm","openjdk","sun"],"owner":{"reputation":2322,"user_id":1475346,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/532252b8858ecc94c49665dd44beb916?s=128&d=identicon&r=PG","display_name":"Chasmo","link":"http://stackoverflow.com/users/1475346/chasmo"},"is_answered":false,"view_count":6,"answer_count":0,"score":0,"last_activity_date":1427827110,"creation_date":1427827110,"question_id":29376004,"link":"http://stackoverflow.com/questions/29376004/what-is-the-purpose-of-sun-jvm-hotspot-helloworld","title":"What is the purpose of sun.jvm.hotspot.HelloWorld?"},{"tags":["java","recursion","binary-search-tree"],"owner":{"reputation":41,"user_id":3491653,"user_type":"registered","accept_rate":100,"profile_image":"https://www.gravatar.com/avatar/ab37f26e0508ac72bdee8c31e022782b?s=128&d=identicon&r=PG&f=1","display_name":"user3491653","link":"http://stackoverflow.com/users/3491653/user3491653"},"is_answered":false,"view_count":4,"answer_count":0,"score":0,"last_activity_date":1427827100,"creation_date":1427827100,"question_id":29375999,"link":"http://stackoverflow.com/questions/29375999/count-number-of-insertions-in-bst","title":"Count number of Insertions in BST"},{"tags":["c++","opencv","mat","iplimage"],"owner":{"reputation":1,"user_id":4734929,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/2994d61ee862f235d009c8a26af4063d?s=128&d=identicon&r=PG&f=1","display_name":"Nicolazz92","link":"http://stackoverflow.com/users/4734929/nicolazz92"},"is_answered":false,"view_count":2,"answer_count":0,"score":0,"last_activity_date":1427827093,"creation_date":1427827093,"question_id":29375995,"link":"http://stackoverflow.com/questions/29375995/c-opencv-2-4-how-to-equalizehist-an-iplimage-type","title":"c++ + OpenCV 2.4: How to equalizeHist an IplImage* type?"},{"tags":["java","maven","maven-3","tika","apache-tika"],"owner":{"reputation":629,"user_id":1237617,"user_type":"registered","accept_rate":72,"profile_image":"https://www.gravatar.com/avatar/751d2ec3098e48dbe032d9f533c158a6?s=128&d=identicon&r=PG","display_name":"Jakub Bochenski","link":"http://stackoverflow.com/users/1237617/jakub-bochenski"},"is_answered":false,"view_count":1,"answer_count":0,"score":0,"last_activity_date":1427827069,"creation_date":1427827069,"question_id":29375988,"link":"http://stackoverflow.com/questions/29375988/transitive-test-provided-dependency-is-included-in-my-project","title":"Transitive test/provided dependency is included in my project"},{"tags":["java","floating-point"],"owner":{"reputation":15,"user_id":3163857,"user_type":"registered","accept_rate":75,"profile_image":"https://www.gravatar.com/avatar/712b5cf00e55cbb8f9049817765e7fb1?s=128&d=identicon&r=PG&f=1","display_name":"skumar","link":"http://stackoverflow.com/users/3163857/skumar"},"is_answered":false,"view_count":9,"answer_count":2,"score":0,"last_activity_date":1427827317,"creation_date":1427826987,"question_id":29375965,"link":"http://stackoverflow.com/questions/29375965/float-parsefloat-for-negative-number-out-of-range-returns-0-0","title":"Float.parseFloat for negative number out of range returns 0.0"},{"tags":["c++","registry"],"owner":{"reputation":6,"user_id":4704983,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/5f9d390c007018a8b9c1b503767510f0?s=128&d=identicon&r=PG&f=1","display_name":"CBMC","link":"http://stackoverflow.com/users/4704983/cbmc"},"is_answered":false,"view_count":13,"answer_count":0,"score":0,"last_activity_date":1427826961,"creation_date":1427826961,"question_id":29375961,"link":"http://stackoverflow.com/questions/29375961/c-add-to-registry","title":"c++ add to registry"},{"tags":["jquery","html","image","gallery","next"],"owner":{"reputation":17,"user_id":3355407,"user_type":"registered","accept_rate":71,"profile_image":"http://graph.facebook.com/577861199/picture?type=large","display_name":"Jeen","link":"http://stackoverflow.com/users/3355407/jeen"},"is_answered":false,"view_count":7,"answer_count":0,"score":0,"last_activity_date":1427826953,"creation_date":1427826953,"question_id":29375958,"link":"http://stackoverflow.com/questions/29375958/jquery-next-button-not-working-with-image-gallery","title":"Jquery next button not working with image gallery."},{"tags":["php","multithreading","log4php"],"owner":{"reputation":579,"user_id":595305,"user_type":"registered","accept_rate":84,"profile_image":"https://www.gravatar.com/avatar/de73cca7e15d1d8feb243f3c125dc3ca?s=128&d=identicon&r=PG","display_name":"mike rodent","link":"http://stackoverflow.com/users/595305/mike-rodent"},"is_answered":false,"view_count":5,"answer_count":0,"score":0,"last_activity_date":1427826920,"creation_date":1427826920,"question_id":29375949,"link":"http://stackoverflow.com/questions/29375949/log4php-output-with-loggerlayouthtml-gets-a-bit-messy","title":"log4php output with LoggerLayoutHtml gets a bit messy"},{"tags":["javascript","html","asp.net-mvc","knockout.js"],"owner":{"reputation":5,"user_id":4625406,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/4bbcc871d6e4325273635813b7e53ca2?s=128&d=identicon&r=PG&f=1","display_name":"Obviously","link":"http://stackoverflow.com/users/4625406/obviously"},"is_answered":false,"view_count":6,"answer_count":0,"score":0,"last_activity_date":1427826912,"creation_date":1427826912,"question_id":29375944,"link":"http://stackoverflow.com/questions/29375944/how-to-declare-variable-inline-in-knockout","title":"How to declare variable inline in Knockout"},{"tags":["javascript","cookies","session-cookies"],"owner":{"reputation":19,"user_id":4603451,"user_type":"registered","accept_rate":100,"profile_image":"http://i.stack.imgur.com/rxsa9.jpg?s=128&g=1","display_name":"opengl","link":"http://stackoverflow.com/users/4603451/opengl"},"is_answered":false,"view_count":16,"answer_count":1,"score":0,"last_activity_date":1427827300,"creation_date":1427826877,"question_id":29375938,"link":"http://stackoverflow.com/questions/29375938/cookie-not-being-deleted-after-session-closed","title":"Cookie not being deleted after session closed"},{"tags":["javascript","ecmascript-5"],"owner":{"reputation":79,"user_id":2909492,"user_type":"registered","accept_rate":88,"profile_image":"https://www.gravatar.com/avatar/1613b79efeeb1216a156dd86a205ecc7?s=128&d=identicon&r=PG&f=1","display_name":"beli","link":"http://stackoverflow.com/users/2909492/beli"},"is_answered":false,"view_count":9,"answer_count":0,"score":0,"last_activity_date":1427826853,"creation_date":1427826853,"question_id":29375931,"link":"http://stackoverflow.com/questions/29375931/why-is-typev-value-of-calling-normal-function-declaration-not-reference-in","title":"Why is `Type(V)` value of calling normal function declaration not `reference` in `ECMA-262 5.1`?"},{"tags":["javascript","graphics","smoothing","curves"],"owner":{"reputation":123,"user_id":2386836,"user_type":"registered","accept_rate":78,"profile_image":"https://www.gravatar.com/avatar/d67924e6b62cc61114a38493dff3a576?s=128&d=identicon&r=PG","display_name":"Wes Modes","link":"http://stackoverflow.com/users/2386836/wes-modes"},"is_answered":false,"view_count":9,"answer_count":0,"score":0,"last_activity_date":1427826751,"creation_date":1427826751,"question_id":29375908,"link":"http://stackoverflow.com/questions/29375908/generating-river-meanders-in-javascript","title":"Generating River Meanders in Javascript"},{"tags":["java","eclipse"],"owner":{"reputation":291,"user_id":1696191,"user_type":"registered","accept_rate":78,"profile_image":"http://i.stack.imgur.com/73J7c.jpg?s=128&g=1","display_name":"the.wizard","link":"http://stackoverflow.com/users/1696191/the-wizard"},"is_answered":false,"view_count":23,"answer_count":1,"score":0,"last_activity_date":1427827066,"creation_date":1427826713,"question_id":29375898,"link":"http://stackoverflow.com/questions/29375898/why-is-my-java-file-looks-strange-in-eclipse","title":"Why Is My Java File Looks Strange in Eclipse?"},{"tags":["javascript","asp.net","jquery-ui","datepicker","jquery-ui-datepicker"],"owner":{"reputation":73,"user_id":2208721,"user_type":"registered","accept_rate":82,"profile_image":"https://www.gravatar.com/avatar/1821de07244119f8e1ee7be8a9bbb5cd?s=128&d=identicon&r=PG","display_name":"Rainhider","link":"http://stackoverflow.com/users/2208721/rainhider"},"is_answered":false,"view_count":5,"answer_count":0,"score":0,"last_activity_date":1427826690,"creation_date":1427826690,"question_id":29375894,"link":"http://stackoverflow.com/questions/29375894/datepicker-setting-date-does-not-trigger-beforeshowday","title":"Datepicker Setting Date does not trigger beforeshowday"},{"tags":["javascript","jquery","css","overflow","hidden"],"owner":{"reputation":1,"user_id":4733916,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/1c5c90f234b886317c7b1ed5d47831d8?s=128&d=identicon&r=PG&f=1","display_name":"Mr Glass","link":"http://stackoverflow.com/users/4733916/mr-glass"},"is_answered":false,"view_count":13,"answer_count":1,"score":0,"last_activity_date":1427827149,"creation_date":1427826689,"question_id":29375892,"link":"http://stackoverflow.com/questions/29375892/determine-what-text-is-hidden-due-to-overflow-hidden","title":"Determine What Text Is Hidden Due To overflow: hidden"},{"tags":["c++","architecture","layer","software-design"],"owner":{"reputation":92,"user_id":4117401,"user_type":"registered","accept_rate":44,"profile_image":"https://www.gravatar.com/avatar/d2587eb6dded16ec45391bb7e383e0ae?s=128&d=identicon&r=PG&f=1","display_name":"Marcel Hoffmann","link":"http://stackoverflow.com/users/4117401/marcel-hoffmann"},"is_answered":false,"view_count":13,"answer_count":0,"score":0,"last_activity_date":1427826751,"creation_date":1427826616,"last_edit_date":1427826751,"question_id":29375873,"link":"http://stackoverflow.com/questions/29375873/what-is-the-way-to-separate-layers","title":"What is the way to separate layers"},{"tags":["java","reflection","mockito"],"owner":{"reputation":1,"user_id":4735226,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/41474332c694acd04d9c4cf9846928f5?s=128&d=identicon&r=PG&f=1","display_name":"Arnolddddddd","link":"http://stackoverflow.com/users/4735226/arnolddddddd"},"is_answered":false,"view_count":13,"answer_count":0,"score":0,"last_activity_date":1427826615,"creation_date":1427826615,"question_id":29375869,"link":"http://stackoverflow.com/questions/29375869/a-java-function-to-generate-mock-data-code-by-taking-a-java-object","title":"A Java function to Generate Mock data CODE by taking a Java Object"},{"tags":["javascript","html","button","dojo","programmatically-created"],"owner":{"reputation":8,"user_id":2840472,"user_type":"registered","accept_rate":25,"profile_image":"https://www.gravatar.com/avatar/3aedc4a04494e35b32fa0c3b5bb30b2e?s=128&d=identicon&r=PG&f=1","display_name":"okorng","link":"http://stackoverflow.com/users/2840472/okorng"},"is_answered":false,"view_count":4,"answer_count":0,"score":0,"last_activity_date":1427826604,"creation_date":1427826604,"question_id":29375866,"link":"http://stackoverflow.com/questions/29375866/dojo-programmatic-button-shorter-than-declarative-button","title":"Dojo programmatic Button shorter than declarative Button?"},{"tags":["c++","corba"],"owner":{"reputation":279,"user_id":469203,"user_type":"registered","accept_rate":72,"profile_image":"https://www.gravatar.com/avatar/b05eb3cabc3775c971ca05d3da9990e9?s=128&d=identicon&r=PG","display_name":"tadpole","link":"http://stackoverflow.com/users/469203/tadpole"},"is_answered":false,"view_count":5,"answer_count":0,"score":0,"last_activity_date":1427827227,"creation_date":1427826603,"last_edit_date":1427827227,"question_id":29375865,"link":"http://stackoverflow.com/questions/29375865/output-file-not-created-when-running-server-under-imr","title":"Output file not created when running server under IMR"},{"tags":["c","struct","segmentation-fault","stack"],"owner":{"reputation":8,"user_id":3955458,"user_type":"registered","profile_image":"https://www.gravatar.com/avatar/698f5c52422ca73eb4606bff43c6bdff?s=128&d=identicon&r=PG&f=1","display_name":"mikefedi","link":"http://stackoverflow.com/users/3955458/mikefedi"},"is_answered":false,"view_count":29,"answer_count":1,"score":1,"last_activity_date":1427826905,"creation_date":1427826598,"question_id":29375859,"link":"http://stackoverflow.com/questions/29375859/c-segmentation-fault-with-struct","title":"C - Segmentation fault with struct"},{"tags":["c++","c++11"],"owner":{"reputation":998,"user_id":2843835,"user_type":"registered","accept_rate":75,"profile_image":"https://www.gravatar.com/avatar/4636d7abd5b19dcee891618859eaa5ee?s=128&d=identicon&r=PG&f=1","display_name":"Mark","link":"http://stackoverflow.com/users/2843835/mark"},"is_answered":false,"view_count":19,"answer_count":1,"score":1,"last_activity_date":1427827254,"creation_date":1427826564,"question_id":29375844,"link":"http://stackoverflow.com/questions/29375844/intercepting-sleep-calls-in-c","title":"Intercepting sleep calls in C++"}]'
__r = 340000